<?php

namespace Ardent\UtilBundle\EntityBuilder;

use Doctrine\ORM\EntityManagerInterface;
use Exception;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ChoiceQuestion;
use Symfony\Component\Console\Question\ConfirmationQuestion;
use Symfony\Component\Console\Style\SymfonyStyle;

abstract class AbstractEntityCommand extends Command
{
    public function __construct(
        protected readonly EntityManagerInterface $em,
    )
    {
        parent::__construct();
    }

    abstract function getConfig(Builder $builder): void;

    abstract function getEntityClass(): string;

    protected function configure(): void
    {
        $this
            ->addArgument('id', InputArgument::OPTIONAL, 'Object Id')
            ->addOption('create', 'c', InputOption::VALUE_NONE, 'Create entity')
            ->addOption('modify', 'm', InputOption::VALUE_NONE, 'Edit entity')
            ->addOption('delete', 'd', InputOption::VALUE_NONE, 'Delete entity')
            ->addOption('show', 's', InputOption::VALUE_NONE, 'Show entity')
            ->addOption('all', 'a', InputOption::VALUE_NONE, 'Show all entities');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $id = $input->getArgument('id');

        $builder = new Builder($io);
        $this->getConfig($builder);

        if ($input->getOption('create')) {
            do {
                $io->info(sprintf('Creating new "%s"', $this->getEntityClass()));
                $entity = new ($this->getEntityClass())();
                $builder->setAll($entity);
                $this->em->persist($entity);

                if ($io->askQuestion(new ConfirmationQuestion(sprintf('Are you sure you want to create "%s" with above properties', $entity)))) {
                    $io->success(sprintf('Object "%s" stored successfully', $entity));
                }
            } while ($io->askQuestion(new ConfirmationQuestion('Create another entity?', false)));
        } elseif ($input->getOption('modify')) {
            $io->info(sprintf('Modifying "%s"', $this->getEntityClass()));
            $entity = $this->select($io, $id);
            do {
                $builder->setByQuestion($entity);
            } while ($io->askQuestion(new ConfirmationQuestion('Modify another property?')));
            $this->em->persist($entity);
        } elseif ($input->getOption('delete')) {
            $io->info(sprintf('Deleting "%s"', $this->getEntityClass()));
            $entity = $this->select($io, $id);
            if ($io->askQuestion(new ConfirmationQuestion(sprintf('Are you sure you want to delete %s', $entity)))) {
                $name = (string)$entity;
                $this->em->remove($entity);
                $io->success(sprintf('Config "%s" deleted', $name));
            }
        } elseif ($input->getOption('show')) {
            $io->info(sprintf('Showing "%s"', $this->getEntityClass()));
            $entity = $this->select($io, $id);
            $table = $io->createTable();
            $table->setHeaders(['property', 'value']);
            $table->setRows($builder->getAllPropertiesTable($entity));
            $table->setHeaderTitle(sprintf('id: %s', $entity->getId()));
            $table->render();
        } elseif ($input->getOption('all')) {
            $io->info(sprintf('Showing all %s', $this->getEntityClass()));
            $entities = $this->em->getRepository($this->getEntityClass())->findAll();
            $table = $io->createTable();
            $table->setHeaders($builder->getPopertyNames());
            $table->setRows(array_map(fn(object $o) => $builder->getAllProperties($o), $entities));
            $table->setHeaderTitle($this->getEntityClass());
            $table->render();
        } else {
            $io->error('Select at least one of the options: Create (-c), Modify (-m), Delete (-d) or Show (-s)');
            return Command::INVALID;
        }

        $this->em->flush();

        return Command::SUCCESS;
    }

    private function select(SymfonyStyle $io, ?int $id): object
    {
        $repo = $this->em->getRepository($this->getEntityClass());
        if ($id) {
            $object = $repo->find($id);
            if ($object) {
                return $object;
            }
        }
        $entities = $repo->findAll();

        if (empty($entities)) {
            throw new Exception('No objects found');
        }
        $question = new ChoiceQuestion('Select configuration to delete', $entities);
        return $io->askQuestion($question);
    }
}